#include "stdafx.h"
#include "SettingDlg.h"
#include "App.h"
#define TIMER_SPEAKER 1
#define TIMER_MIC     2

SettingConfig  SettingDlg::m_settingConfig = SettingConfig();

SettingDlg::SettingDlg():SHostDialog(_T("LAYOUT:XML_SETTING"))
{
	m_bInited = false;
	m_soundcurtime = 0;
}


SettingDlg::~SettingDlg()
{
	::KillTimer(m_hWnd, TIMER_MIC);
}

void SettingDlg::OnClose()
{
	App::video()->stopDeviceTest();
	App::audio()->stopSpeakerTest();
	App::sdk()->setLocalVideoMirror(m_settingConfig.isMirror);
	App::audio()->setSpeaker(m_settingConfig.spkGuid);
	OnCancel();
}

void SettingDlg::OnOK()
{
	int camCurSel = m_camComBox->GetCurSel();
	if (camCurSel >=0)
	{
		m_settingConfig.camGuid = m_videoIdList[camCurSel];
		App::video()->stopDeviceTest();
		App::video()->setVideo(m_settingConfig.camGuid);
	}
	int micCurSel = m_micComBox->GetCurSel();
	if (micCurSel >= 0)
	{
		m_settingConfig.micGuid = m_micIdList[micCurSel];
		App::audio()->setMicrophone(m_settingConfig.micGuid);
	}
	int spkCurSel = m_spkComBox->GetCurSel();
	if (spkCurSel >= 0)
	{
		m_settingConfig.spkGuid = m_spkIdList[spkCurSel];
		App::audio()->setSpeaker(m_settingConfig.spkGuid);
	}

	SSliderBar *sliderSound = FindChildByName2<SSliderBar>("slider_sound");
	int spkSound =  sliderSound->GetValue();
	App::audio()->setSpeakerVolume(spkSound * 255 / 100);
	SCheckBox *checkbox =  FindChildByName2<SCheckBox>("check_mirror");
	m_settingConfig.isMirror = checkbox->IsChecked();
	App::sdk()->setLocalVideoMirror(m_settingConfig.isMirror);
	App::audio()->stopMicrophoneTest();
	App::audio()->stopSpeakerTest();
	SHostDialog::OnOK();
}

void SettingDlg::OnMusicPlay()
{
	SButton * btn_musicplay = FindChildByName2<SButton>("btn_musicplay");
	SButton * btn_musicpause = FindChildByName2<SButton>("btn_musicpause");
	SASSERT(btn_musicplay);
	SASSERT(btn_musicpause);
	btn_musicpause->SetVisible(TRUE);
	btn_musicplay->SetVisible(FALSE);
	std::string detectSpeakerPath = S_T2UTF8(App::appDir()).GetBuffer(0);
	detectSpeakerPath += "\\detectionDevice_default.wav";
	App::audio()->startSpeakerTest(detectSpeakerPath);
	::SetTimer(m_hWnd, TIMER_SPEAKER,1000,NULL);
	m_soundcurtime = 0;
	SWindow::Invalidate();
}

void SettingDlg::OnMusicPause()
{
	SButton * btn_musicplay = FindChildByName2<SButton>("btn_musicplay");
	SButton * btn_musicpause = FindChildByName2<SButton>("btn_musicpause");
	SStatic * pEdit = FindChildByName2<SStatic>("txt_sound_cur_time");
	SASSERT(btn_musicplay);
	SASSERT(btn_musicpause);
	SASSERT(pEdit);
	btn_musicpause->SetVisible(FALSE);
	btn_musicplay->SetVisible(TRUE);
	App::audio()->stopSpeakerTest();
	::KillTimer(m_hWnd,TIMER_SPEAKER);
	m_soundcurtime = 0;
	pEdit->SetWindowText(_T("0:00"));
	SWindow::Invalidate();
}

void SettingDlg::OnCheckBoxMirror()
{
	SCheckBox *checkbox = FindChildByName2<SCheckBox>("check_mirror");
	if (checkbox->IsChecked())
	{
		App::sdk()->setLocalVideoMirror(true);
	}
	else
	{
		App::sdk()->setLocalVideoMirror(false);
	}
	
}

BOOL SettingDlg::SliderSoundEvent(EventArgs *pEvt)
{
	EventSliderPos *pEvt1 = dynamic_cast<EventSliderPos*>(pEvt);
	SSliderBar *sliderSound = FindChildByName2<SSliderBar>("slider_sound") ;
	SStatic *currentPos = FindChildByName2<SStatic>("text_currentsound");

	if (pEvt1)
	{
		if (pEvt1->sender == sliderSound)
		{
			int current_pos = sliderSound->GetValue();
			// process data
			SStringT tempText;
			tempText.Format(_T("%d"),current_pos);
			App::audio()->setSpeakerVolume(current_pos * 255 / 100);
			currentPos->SetWindowTextW(tempText);
		}
	}
	return __super::_HandleEvent(pEvt);
}

void SettingDlg::OnCbVideoInputChange(EventArgs *pEvt)
{
	if (!m_bLayoutInited)
		return;
	EventCBSelChange *e2 = sobj_cast<EventCBSelChange>(pEvt);
	SComboBox *pCbx = sobj_cast<SComboBox>(e2->sender);
	if (e2->nCurSel !=-1)
	{
		SStringT str = pCbx->GetLBText(e2->nCurSel);
		if (e2->nCurSel < m_videoIdList.size())
		{
			std::string camGuid = m_videoIdList[e2->nCurSel];
			SRealWnd *testVideo = FindChildByName2<SRealWnd>("test_video");
			VideoCanvas canvas;
			canvas.view = (void *)testVideo->GetRealHwnd();
			canvas.renderMode = TK_RENDER_MODE_ADAPTIVE;
			App::video()->stopDeviceTest();
			int ret = App::video()->startDeviceTest(camGuid, canvas);
			if (ret)
			{
				printf("startDeviceTest ret:%d\n", ret);
			}
		}
	}
}

void SettingDlg::OnCbAudioInputChange(EventArgs *pEvt)
{
	if (!m_bLayoutInited)
		return;
	EventCBSelChange *e2 = sobj_cast<EventCBSelChange>(pEvt);
	SComboBox *pCbx = sobj_cast<SComboBox>(e2->sender);
	if (e2->nCurSel != -1)
	{
		//?mic
		SStringT str = pCbx->GetLBText(e2->nCurSel);
		if (e2->nCurSel < m_micIdList.size())
		{
			App::audio()->stopMicrophoneTest();
			App::audio()->startMicrophoneTest();
		}
	}
}

void SettingDlg::OnCbAudioOutputChange(EventArgs *pEvt)
{
	if (!m_bLayoutInited)
		return;
	EventCBSelChange *e2 = sobj_cast<EventCBSelChange>(pEvt);
	SComboBox *pCbx = sobj_cast<SComboBox>(e2->sender);
	if (e2->nCurSel != -1)
	{
		SStringT str = pCbx->GetLBText(e2->nCurSel);
		if (e2->nCurSel < m_spkIdList.size())
		{
			std::string spkGuid = m_spkIdList[e2->nCurSel];
			App::audio()->setSpeaker(spkGuid);
		}
	}
}

void SettingDlg::OnTimer(char cTimerID)
{
	switch (cTimerID)
	{
	case 1:
	{
		SStatic * pEdit = FindChildByName2<SStatic>("txt_sound_cur_time");
		m_soundcurtime++;
		SStringT curtimeStr;
		if (m_soundcurtime <14)
		{
			curtimeStr.Format(_T("0:%02d"), m_soundcurtime);
			pEdit->SetWindowText(curtimeStr);
		}
		else
		{
			OnMusicPause();
		}
		
	}
	break;
	case 2:
	{
		int micVolume = App::audio()->getMicrophoneVolume();
		SStatic *currentPos = FindChildByName2<SStatic>("text_currentsound");
		SStringT tempText;
		tempText.Format(_T("%d"), (int)(micVolume*100/255));
		currentPos->SetWindowTextW(tempText);
	}
	break;
	default:
		break;
	}
}

void SettingDlg::InitDeviceInfo()
{
	m_videoIdList.clear();
	TKVideo* videoDev = App::video();
	int camCount = videoDev->getVideoCount();
	m_camComBox->ResetContent();
	for (int i = 0; i<camCount ; i++)
	{
		std::string name, guid;
		videoDev->getVideo(i, name, guid);
		m_videoIdList.push_back(guid);
		m_camComBox->InsertItem(i, S_UTF82T(name.data()), 0, 0);
		if (m_settingConfig.camGuid == guid)
		{
			m_camComBox->SetCurSel(i);
		}
	}
	if (m_camComBox->GetCurSel() <0)
	{
		m_camComBox->SetCurSel(0);
	}
	TKAudio* audioDev = App::audio();
	int micCount = audioDev->getMicrophoneCount();
	m_micComBox->ResetContent();
	for (int i = 0; i < micCount; i++)
	{
		std::string name, guid;
		audioDev->getMicrophone(i, name, guid);
		m_micIdList.push_back(guid);
		m_micComBox->InsertItem(i,S_UTF82T(name.data()),0,0);
		if (m_settingConfig.micGuid == guid)
		{
			m_micComBox->SetCurSel(i);
		}
	}
	if (m_micComBox->GetCurSel() <0)
	{
		m_micComBox->SetCurSel(0);
	}
	int spkCount = audioDev->getSpeakerCount();
	m_spkComBox->ResetContent();
	for (int i = 0; i < spkCount; i++)
	{
		std::string name, guid;
		audioDev->getSpeaker(i, name, guid);
		m_spkIdList.push_back(guid.data());
		m_spkComBox->InsertItem(i, S_UTF82T(name.data()),0,0);
		if (m_settingConfig.spkGuid == guid)
		{
			m_spkComBox->SetCurSel(i);
		}
	}
	if (m_spkComBox->GetCurSel() <0 )
	{
		m_spkComBox->SetCurSel(0);
	}
	int spkVol = audioDev->getSpeakerVolume();//speaker volume range is [0,255]
	SSliderBar* spkSlider =  FindChildByName2<SSliderBar>("slider_sound");
	spkSlider->SetValue(spkVol * 100 / 255);

	SStringT tempText;
	tempText.Format(_T("%d"), spkSlider->GetValue());
	SStatic *currentPos = FindChildByName2<SStatic>("text_currentsound");
	currentPos->SetWindowTextW(tempText);


	SCheckBox *checkbox = FindChildByName2<SCheckBox>("check_mirror");
	checkbox->SetCheck(m_settingConfig.isMirror);

	videoDev->stopDeviceTest();

	SRealWnd *testVideo = FindChildByName2<SRealWnd>("test_video");
	VideoCanvas canvas;
	canvas.view = (void *)testVideo->GetRealHwnd();
	canvas.renderMode = TK_RENDER_MODE_ADAPTIVE;
	int videoIndex = m_camComBox->GetCurSel();
	if (camCount >0 && videoIndex < m_videoIdList.size())
	{
		
		int ret =  videoDev->startDeviceTest(m_videoIdList[videoIndex],canvas);
		if (ret)
		{
			printf("startDeviceTest ret:%d\n",ret);
		}
	}

}

void SettingDlg::SetConfig(SettingConfig config)
{
	m_settingConfig = config;
}

BOOL SettingDlg::OnInitDialog(HWND wndFocus, LPARAM lInitParam)
{
	
	m_camComBox = FindChildByName2<SComboBox>("cb_videoinput");
	m_micComBox = FindChildByName2<SComboBox>("cb_audioinput");
	m_spkComBox = FindChildByName2<SComboBox>("cb_audiooutput");
	SASSERT(m_camComBox);
	SASSERT(m_micComBox);
	SASSERT(m_spkComBox);
	InitDeviceInfo();
	::SetTimer(m_hWnd, TIMER_MIC, 100, NULL);
	m_bLayoutInited = true;
	return TRUE;
}
