#ifndef TK_AUDIO_VIDEO_DEVICE_H_
#define TK_AUDIO_VIDEO_DEVICE_H_
#include "tkdefinition.h"
class TKAudio
{
public:
	/**
	* get recording device count
	* @return return the count
	*/
	virtual int getMicrophoneCount() = 0;

	/**
	* get recording device name and id by index
	* @param index  device index
	* @param [out] strNameUTF8   device name
	* @param [out] deviceId   device id
	*/
	virtual void getMicrophone(int index, std::string& strNameUTF8,std::string& deviceId) = 0;

	/**
	* get current device id
	* @param [out] current_devId
	*/
	virtual void getMicrophone(std::string& deviceId) = 0;

	/**
	* set use recording device id
	* @param [in] deviceId
	*/
	virtual void setMicrophone(const std::string& deviceId) = 0;

	/**
	* get playout device count
	* @return return the count
	*/
	virtual int getSpeakerCount() = 0;

	/**
	* get playout device name and id by index
	* @param index  device index
	* @param [out] strNameUTF8   device name
	* @param [out] deviceId   device id
	*/
	virtual void getSpeaker(int index, std::string& strNameUTF8, std::string& deviceId) = 0;

	/**
	* get current device id
	* @param [out] current_devId
	*/
	virtual void getSpeaker(std::string& deviceId) = 0;
	
	/**
	* set use playout device id
	* @param [in] deviceId
	*/
	virtual void setSpeaker(const std::string& deviceId) = 0;

	/**
	* set use playout device volume
	* @param [in] volume 0255
	*/
	virtual void setSpeakerVolume(int volume) = 0;

	/**
	* get current playout device volume
	* @return return current volume 0255
	*/
	virtual int getSpeakerVolume() = 0;

	/**
	* set use recording device volume
	* @param [in] volume 0255
	*/
	virtual void setMicrophoneVolume(int volume) = 0;

	/**
	* get current recording device volume
	* @return return current volume 0255
	*/
	virtual int getMicrophoneVolume() = 0;

	/**
	* set use recording device gain
	* @param [in] gain 0.1
	*/
	virtual void setMicrophoneGain(float gain) = 0;

	/**
	* get current recording device gain
	* @return return current gain default 1.0
	*/
	virtual float getMicrophoneGain() = 0;

	/**
	* mute recording device 
	* @param [in] bMute 
	*/
	virtual void muteMic(bool bMute) = 0;
	
	/**
	* current recording device mute status
	* @return 
	*/
	virtual bool isMicMuted() = 0;
	
	/**
	* test the recording audio device to know whether it can worked well
	* @param loopBack if true playback
	* @return 0
	*/
	virtual int startMicrophoneTest(bool loopBack = true) = 0;

	/**
	* stop test the recording audio device
	* @param loopBack if true playback
	* @return 0
	*/
	virtual int stopMicrophoneTest() = 0;

	/**
	* test the playback audio device to know whether it can worked well
	* @param [in] testAudioFilePath
	*        the path of the .wav file
	* @return return 0 if success and you can hear the sound of the .wav file or an error code.
	*/
	virtual int startSpeakerTest(const std::string& testAudioFilePath) = 0;

	/**
	* stop the playback audio device testing
	* @return return 0 if success or an error code
	*/
	virtual int stopSpeakerTest() = 0;

	/**
	* set the Automatic Gain Control status
	* @param enable  on/off  default on
	* @return return 0 if success or an error code
	*/
	virtual int setAgcStatus(bool enable) = 0;

	/**
	* set the Acoustic Echo Chancellor status
	* @param enable  on/off default on
	* @return return 0 if success or an error code
	*/
	virtual int setAecStatus(bool enable) = 0;

	/**
	* set the Automatic Noise Suppression status
	* @param enable  on/off default on
	* @return return 0 if success or an error code
	*/
	virtual int setNsStatus(bool enable) = 0;
};

class TKVideo
{
public:
	/**
	* get video device count
	* @return return the count
	*/
	virtual int getVideoCount() = 0;

	/**
	* get video device name and id by index
	* @param index  video index
	* @param [out] cameraNameUTF8  video device name
	* @param [out] deviceId  video device id
	*/
	virtual void getVideo(int index, std::string& cameraNameUTF8, std::string& deviceId) = 0;

	/**
	* set use video device id
	* @param [in] deviceId
	*/
	virtual void setVideo(const std::string& deviceId) = 0;

	/**
	* get current video device id
	* @param [out] current_devId 
	*/
	virtual void getVideo(std::string& current_devId) = 0;

	/**
	* test the video capture device to know whether it can worked well
	* @param deviceId  
	* @param [in] hwnd
	*        the HWND of the video-display window
	* @return return 0 if success or an error code
	*/
	virtual int startDeviceTest(const std::string& deviceId, const VideoCanvas& view) = 0;

	/**
	* stop the video device testing
	* @return return 0 if success or an error code
	*/
	virtual int stopDeviceTest() = 0;
};

#endif //TK_AUDIO_VIDEO_DEVICE_H_
