#ifndef TK_MEDIACAPTURER_H_
#define TK_MEDIACAPTURER_H_
#include "tkdefinition.h"
#include "../utils/scoped_ref_ptr.h"
#include "../utils/refcountinterface.h"

// this class should be implemented by user
// you can start capture video data after the start method be invoked,
// then push video data to sdk though pushFrame method which in TKMediaFactory
// eg for tk::scoped_refptr:
//    class ExternalMediaSource : public TKMediaSource
//    tk::scoped_refptr<ExternalMediaSource> external_source_ = new tk::RefCountedObject<ExternalMediaSource>();
class TKMediaSource : public tk::RefCountInterface
{
public:
	TKMediaSource(std::string sourceId) { source_id_ = sourceId; };
	virtual ~TKMediaSource() {};
	std::string sourceId() { return source_id_; };
	virtual void start() = 0;
	virtual void stop() = 0;
	virtual bool isRunning() = 0;
	virtual VideoProfile getVideoProfile() { return VideoProfile(0, 0, 0); };
private:
	std::string source_id_;
};

// implemented by sdk
// get instance through getInterface method with type TK_IID_MEDIA_FACTORY
class TKMediaFactory
{
public:
	virtual int addMediaSource(tk::scoped_refptr<TKMediaSource> source) = 0;
	virtual int removeMediaSource(tk::scoped_refptr<TKMediaSource> source) = 0;
	virtual int pushFrame(std::string sourceId, const VideoFrame &frame) = 0;
};
#endif // !TK_MEDIACAPTURER_H_


