/*****************************************************************************
* tkroommanager.h: tkroomsdk public header
*****************************************************************************
* v3.2.3-20190402
*****************************************************************************/

#ifndef TK_ROOMMANAGER_H_
#define TK_ROOMMANAGER_H_
#include "tkglobal.h"
#include "tkdefinition.h"
#include "api/tkroomuser.h"
#include "utils/variant.h"
#include "utils/scoped_ref_ptr.h"
class TKRoomManagerObserver;
class TKMediaFrameObserver;
class TKRoomManager
{
public:
	/**
	* init sdk
	* @appKey authKey from  management system
	* @options  TKMULTILOCALSTREAM  TKMAXRECONNECTCOUNT
	* @return 0 success
	*/
	virtual int init(const std::string& appKey, const tk::VariantMap& options = tk::VariantMap()) = 0;

	/**
	* deinitialize sdk
	* @return 0 success
	*/
	virtual int uninit() = 0;
	
	/**
	* set logger info
	* @info loglevel logpath limitsize
	* @return 0 success
	*/
	virtual void setLoggerInfo(const LoggerInfo& info) = 0;
	
	/**
	* set RoomManager observer
	* @observer event observer
	* @return 0 success
	*/
	virtual void registerRoomObserver(TKRoomManagerObserver* observer) = 0;

	/**
	* set media observer
	* @observer media frame observer
	* @return 0 success
	*/
	virtual void registerMediaObserver(TKMediaFrameObserver* observer) = 0;

	virtual int getInterface(TK_INTERFACE_TYPE interfaceID, void** inter) = 0;
	
	//÷
	virtual int setRoomProfile(TK_ROOM_PROFILE profile) = 0;
	/**
	* switch room to only audio or video and audio ,receive onAudioRoomSwitch callback (multi local stream invalid)
	* @param [in] onlyAudio
	*/
	virtual int switchOnlyAudioRoom(bool onlyAudio) = 0;

	/**
	* join room 
	* @param host webserver addr  global.talk-cloud.net
	* @param port webserver port  if TKUSESECURESOCKET true 443 else 80 
	* @param nickname  the nickname of user in room
	* @param third_uid  the unique id of user in room
	* @param roomjsonparams TKROOMID  the number of room,this id must be created by talk-cloud management system.
	*                       TKROOMPWD  room password 
	* @param userjsonparams  the customized properties of user in room. json format
	* @return zero if successful, a nonzero value otherwise
	*/
	virtual int joinRoom(const std::string& host
		, const int          port
		, const std::string& nickname
		, tk_uid third_uid
		, const tk::VariantMap& roomjsonparams
		, const tk::VariantMap& userjsonparams = tk::VariantMap()) = 0;

	/**
	* join room with any roomId, if not exist will be created immediately
	* @param roomId  the number of room,this id user can customize.
	* @param nickname  the nickname of user in room
	* @param third_uid  the unique id of user in room
	* @param userjsonparams  the customized properties of user in room. json format
	* @return zero if successful, a nonzero value otherwise
	*/
	virtual int joinRoomEx(const std::string& roomId
		, const std::string& nickname
		, tk_uid third_uid
		, const tk::VariantMap& userjsonparams = tk::VariantMap()) = 0;

	/**
	* leave room
	* @return 0 success
	*/
	virtual int leaveRoom() = 0;

	// ÷ϸ
	// ڽ뷿ɹ
	// ֵ  json param key-value
	virtual tk::VariantMap getRoomProperties() = 0;
	/**
	* set video profile (multi stream local invalid)
	* @profile width height fps
	* @return 0 success
	*/
	virtual int setVideoProfile(const VideoProfile& profile) = 0;
	/**
	* set video profile by device id (only multi local stream valid)
	* @profile width height fps
	* @return 0 success
	*/
	virtual int setVideoProfile(const VideoProfile& profile , const std::string& deviceId) = 0;
	/**
	* set local video render mirror
	* @mirror true mirror effect
	* @return 0 success
	*/
	virtual int setLocalVideoMirror(bool mirror) = 0;

	/**
	* change user property
	* @peerId user id 
	* @tellWhom Who is the msg to 
	* @key change property`s key
	* @value change the key to this value
	* @return 0 success
	*/
	virtual int changeUserProperty(tk_uid peerId
		, tk_uid tellWhom
		, const std::string& key
		, const tk::Variant& value) = 0;

	/**
	* start publish video stream (multi local stream invalid)
	* @return 0 success
	*/
	virtual int publishVideo() = 0;
	/**
	* stop publish video stream (multi local stream invalid)
	* @return 0 success
	*/
	virtual int unpublishVideo() = 0;

	/**
	* start publish video stream by video device id or third video source id
	* the video device id is only allowed in multi-stream mode
	* multi-stream mode is turned on while called the TKRoomManager::init method
	* @sourceId device id or third video source id
	* @return 0 success
	*/
	virtual int publishVideo(const std::string& sourceId) = 0;
	/**
	* stop publish video stream by video device id or third video source id
	* the video device id is only allowed in multi-stream mode
	* multi-stream mode is turned on while called the TKRoomManager::init method
	* @sourceId device id or third video source id
	* @return 0 success
	*/
	virtual int unpublishVideo(const std::string& sourceId) = 0;

	/**
	* start publish audio stream
	* @return 0 success
	*/
	virtual int publishAudio() = 0;
	/**
	* stop publish audio stream
	* @return 0 success
	*/
	virtual int unpublishAudio() = 0;

	/**
	* send text message
	* param toId  receiver ID
	* param message  text message
	* param extendJson the customized properties of message. json format 
	* return 0 success
	*/
	virtual int sendMessage(tk_uid toId, const std::string& message, const tk::VariantMap& extendJson = tk::VariantMap()) = 0;

	// ԶϢ
	// msgNameϢ
	// msgIdϢid
	// toIdҪ֪ͨЩû__allˣ__allExceptSenderԼˣ__allExceptAuditorûˣ__None֪ͨκˣĳûpeerIDֻû
	// dataϢݣNumberStringMapList
	// saveǷ񱣴棬3.5Զ
	// ֵ붨
	virtual int pubMsg(const std::string& msgName
		, const std::string& msgId
		, tk_uid toId
		, const tk::Variant& data
		, tk_uid acassociatedUserID
		, const std::string& acassociatedMsgID
		, uint64_t expireTime) = 0;

	// ɾԶϢ
	// msgNameϢ
	// msgIdϢid
	// toIdҪ֪ͨЩû__allˣ__allExceptSenderԼˣ__allExceptAuditorûˣ__None֪ͨκˣĳûpeerIDֻû
	// dataϢݣNumberStringMapList
	// ֵ붨
	virtual int delMsg(const std::string& msgName
		, const std::string& msgId
		, tk_uid toId
		, const tk::Variant& data = tk::Variant()) = 0;

	/**
	* kick out the user
	* @peerID user id
	* @return 0 success
	*/
	virtual int evictUser(tk_uid peerID) = 0;

	/**
	* start play video by user id (multi local stream invalid)
	* @view render view and renderMode
	* @peerID play user id if empty play my self
	* @return 0 success
	*/
	virtual int playVideo(const VideoCanvas& view,tk_uid peerID) = 0;
	/**
	* stop play video by user id (multi local stream invalid)
	* @peerID user id if empty play my self
	* @return 0 success
	*/
	virtual int unplayVideo(tk_uid peerID) = 0;
	/**
	* start play video by user id 
	* @view render view and renderMode
	* @peerID play user id if empty play my self
	* @sourceId play device id or external video source id
	* @return 0 success
	*/
	virtual int playVideo(const VideoCanvas& view, tk_uid peerID,const std::string& sourceId) = 0;
	/**
	* stop play video by user id and device id 
	* @peerID user id if empty play my self
	* @sourceId device id or external video source id
	* @return 0 success
	*/
	virtual int unplayVideo(tk_uid peerID, const std::string& sourceId) = 0;
	
	/**
	* start play audio by user id
	* @peerID play user id
	* @return 0 success
	*/
	virtual int playAudio(tk_uid peerID) = 0;
	/**
	* stop play audio by user id
	* @peerID user id
	* @return 0 success
	*/
	virtual int unplayAudio(tk_uid peerID) = 0;

	/**
	* set remote user audio volume real time
	* @param [in] peerID
	* @param [in] volume
	*/
	virtual int setRemoteAudioVolume(tk_uid peerID,int volume) = 0;

	/**
	* enable video dual stream mode,big or small
	* @param enable true or false
	* @return 0 success
	*/
	virtual int enableDualStream(bool enable) = 0;
	/**
	* sets video parameters for small streams
	* if you don't set the video parameters of the stream, SDK will automatically adapt the video properties of the stream according to your video stream properties.
	* @param profile video profile
	* @return 0 success
	*/
	virtual int setSmallStreamParameter(const VideoProfile& profile) = 0;
	/**
	* set the default subscription video stream type 
	* if the sender user does not enable dual-stream mode, the default is to receive big streams.
	* @param streamType video stream type TK_VIDEO_STREAM_TYPE
	* @return 0 success
	*/
	virtual int setRemoteDefaultVideoStreamType(TK_VIDEO_STREAM_TYPE streamType) = 0;
	/**
	* set the video stream type for subscription.
	* if the sender user does not enable dual-stream mode, the default is to receive big streams.
	* @param streamType  video stream type TK_VIDEO_STREAM_TYPE
	* @param peerID      user id
	* @param deviceID    device id, enable TKMULTILOCALSTREAM valid
	* @return      0 success
	*/
	virtual int setRemoteVideoStreamType(TK_VIDEO_STREAM_TYPE streamType,tk_uid peerID, const std::string& deviceID = "") = 0;
	
	//ʼýļ
	virtual int startShareMediaFile(const std::string& meidaPath,VideoCanvas canvas) = 0;
	//ֹͣýļ
	virtual int stopShareMediaFile() = 0;
	//ͣýļ
	virtual int pauseMediaFile(bool pause) = 0;
	//϶Ž
	virtual int seekMediaFile(double pos) = 0;
	//ĳûýļ
	virtual int playMediaFile(tk_uid peerID, VideoCanvas canvas) = 0;
	virtual int unplayMediaFile(tk_uid peerID) = 0;

	//ʼ
	virtual int startShareScreen(const DesktopSource& source) = 0;
	//ֹͣ
	virtual int stopShareScreen() = 0;
	//¹
	virtual int updateShareScreen(const DesktopSource& source) = 0;
	//ȡЧ
	virtual int getValidWindowList(std::vector<ValidWindow>& windows, TK_SOURCETYPE type) = 0;
	//ĳûĻ
	virtual int playScreen(tk_uid peerID, VideoCanvas canvas) = 0;
	virtual int unplayScreen(tk_uid peerID) = 0;

	//uflv
	virtual int startRecordScreen(const std::string& file, const DesktopSource& source, const VideoParam& vp = VideoParam()) = 0;
	virtual int stopRecordScreen() = 0;
	virtual int pauseRecordScreen(bool pause) = 0;

	virtual int startRecordAudio(const std::string& mp3) = 0;
	virtual int stopRecordAudio() = 0;
	virtual int pauseRecordAudio(bool pause) = 0;
    
    virtual int startNetworkTest() = 0;
    virtual int stopNetworkTest() = 0;

	/**
	* get server list,can use servername change current server
	* @param [out] list
	* @return 0 success
	*/
	virtual int getServerList(std::vector<ServerInfo>& list) = 0;
	/**
	* change current server by server name
	* @param [in] servername
	* @return 0 success
	*/
	virtual int selectServer(const std::string& servername) = 0;

	virtual void getUserNumberByRoles(const std::vector<int>& roles = std::vector<int>()) = 0;
	virtual void getUserInfoByRoles(int start, int max, const std::vector<int>& roles = std::vector<int>()) = 0;

	virtual int changeUserPropertyByRoles(const std::vector<int>& roles
		, tk_uid tellWhom
		, const std::map<std::string, tk::Variant>& properties) = 0;

	virtual int changeUserPropertyByIds(const std::vector<int>& ids
		, tk_uid tellWhom
		, const std::map<std::string, tk::Variant>& properties) = 0;

	virtual int startServerRecord(TK_SERVER_REC_TYPE recordType = TK_REC_INDEPENDENT_VIDEO, TK_SERVER_REC_CONVERSION conversion = TK_REC_CONVERT_NONE, TK_SERVER_REC_LAYOUT layout = TK_REC_LAYOUT_HOR) = 0;
	virtual int stopServerRecord() = 0;
};

TK_API TK_SDK_EXPORT TKRoomManager* GetRoomClient();
TK_API TK_SDK_EXPORT void         DelRoomClient(TKRoomManager** client);
TK_API TK_SDK_EXPORT const char*    GetSdkVersion();
#endif
