#ifndef TK_VARIANT_H_
#define	TK_VARIANT_H_
#include "tkglobal.h"
namespace tk {
	typedef enum _VariantType {
		V_NULL = 1,
		V_BOOL = 3,
		V_INT8 = 4,
		V_INT16 = 5,
		V_INT32 = 6,
		V_INT64 = 7,
		V_UINT8 = 8,
		V_UINT16 = 9,
		V_UINT32 = 10,
		V_UINT64 = 11,
		V_DOUBLE = 12,
		_V_NUMERIC = 13,
		V_STRING = 17,
		V_BYTEARRAY = 18,
		V_MAP = 19,
		V_ARRAY = 20
	} VariantType;

	class TK_SDK_EXPORT Variant {
	private:
		VariantType _type;
		union {
			bool b;
			int8_t i8;
			int16_t i16;
			int32_t i32;
			int64_t i64;
			uint8_t ui8;
			uint16_t ui16;
			uint32_t ui32;
			uint64_t ui64;
			double d;
			std::string* s;
			std::map<std::string, Variant>* m;
			std::vector<Variant>* l;
		} _value;
		void internalCopy(const Variant &val);
	public:
		Variant();
		Variant(VariantType type);
		Variant(const Variant &val);
		Variant(const bool &val);
		Variant(const int8_t &val);
		Variant(const int16_t &val);
		Variant(const int32_t &val);
		Variant(const int64_t &val);
		Variant(const uint8_t &val);
		Variant(const uint16_t &val);
		Variant(const uint32_t &val);
		Variant(const uint64_t &val);
		Variant(const double &val);
		Variant(const char* val);
		Variant(const std::string &value); 
		Variant(const std::map<std::string, Variant>& val);
		Variant(const std::vector<Variant>& val);
		~Variant();

		Variant & operator=(const Variant &val);
		Variant & operator=(const bool &val);
		Variant & operator=(const int8_t &val);
		Variant & operator=(const int16_t &val);
		Variant & operator=(const int32_t &val);
		Variant & operator=(const int64_t &val);
		Variant & operator=(const uint8_t &val);
		Variant & operator=(const uint16_t &val);
		Variant & operator=(const uint32_t &val);
		Variant & operator=(const uint64_t &val);
		Variant & operator=(const double &val);
		Variant & operator=(const char* val);
		Variant & operator=(const std::string &val);
		Variant & operator=(const std::map<std::string, Variant>& val);
		Variant & operator=(const std::vector<Variant>& val);
	public:
		operator VariantType();
		operator bool();
		operator int8_t();
		operator int16_t();
		operator int32_t();
		operator int64_t();
		operator uint8_t();
		operator uint16_t();
		operator uint32_t();
		operator uint64_t();
		operator double();
		operator std::string();
		operator std::map<std::string, Variant>();
		operator std::vector<Variant>();

		bool operator==(VariantType type);
		bool operator!=(VariantType type);
	public:
		std::map<std::string, Variant> toMap();
		std::vector<Variant> toList();
	public:
		void clear();
		bool isNull();

		static bool fromJson(const std::string& raw, Variant &result);
		bool toJson(std::string& str);
	};
	typedef std::map<std::string, Variant> VariantMap;
}
#endif	/* TK_VARIANT_H_ */

